<?php

namespace Drupal\conductor\Form;

use Drupal\conductor\Service\ConductorHelper;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\Render\Markup;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Module settings form.
 */
class ConductorConfigForm extends ConfigFormBase {

  /**
   * Conductor Helper.
   *
   * @var \Drupal\conductor\Service\ConductorHelper
   */
  protected $conductorHelper;

  /**
   * Messenger.
   *
   * @var \Drupal\Core\Messenger\Messenger
   */
  protected $messenger;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
          $container->get('conductor.helper'),
          $container->get('messenger'),
      );
  }

  /**
   * Constructs class variables.
   *
   * @param \Drupal\conductor\Service\ConductorHelper $conductor_helper
   *   Conductor Helper service.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   Drupal messenger.
   */
  public function __construct(
    ConductorHelper $conductor_helper,
    Messenger $messenger,
  ) {
    $this->conductorHelper = $conductor_helper;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'conductor_config_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Form constructor.
    $form = parent::buildForm($form, $form_state);
    // Default settings.
    $config = $this->config('conductor.settings');

    // API key.
    $form['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Key'),
      '#required' => TRUE,
      '#default_value' => !empty($_ENV['CONDUCTOR_API_KEY']) ? $_ENV['CONDUCTOR_API_KEY'] : $config->get('conductor.api_key'),
      '#description' => $this->t('Get an API key by subscribing and generating one on https://app.conductor.com/u/24936/api-key/'),
    ];

    // Shared secret.
    $form['shared_secret'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Shared Secret'),
      '#required' => TRUE,
      '#default_value' => !empty($_ENV['CONDUCTOR_SHARED_SECRET']) ? $_ENV['CONDUCTOR_SHARED_SECRET'] : $config->get('conductor.shared_secret'),
      '#description' => $this->t('Get an API key by subscribing and generating one on https://app.conductor.com/u/24936/api-key/'),
    ];

    // Content Types.
    $form['content_types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Included Content Types'),
      '#options' => $this->conductorHelper->getContentTypes(),
      '#default_value' => !empty($config->get('conductor.content_types')) ? $config->get('conductor.content_types') : [],
    ];

    // Rank Sources.
    $form['rank_sources'] = [
      '#type' => 'select',
      '#title' => $this->t('Rank Sources'),
      '#options' => $this->conductorHelper->getRankSources(),
      '#default_value' => !empty($config->get('conductor.rank_sources')) ? $config->get('conductor.rank_sources') : 1,
    ];

    // Verbose Logging.
    $form['logging'] = [
      '#type' => 'radios',
      '#title' => $this->t('Verbose Logging'),
      '#options' => [t('Disable'), t('Enable')],
      '#default_value' => !empty($config->get('conductor.logging')) ? $config->get('conductor.logging') : '0',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('conductor.settings');
    $config->set('conductor.api_key', $form_state->getValue('api_key'));
    $config->set('conductor.shared_secret', $form_state->getValue('shared_secret'));
    $config->set('conductor.content_types', $form_state->getValue('content_types'));
    $config->set('conductor.rank_sources', $form_state->getValue('rank_sources'));
    $config->set('conductor.logging', $form_state->getValue('logging'));
    $config->save();

    $account_id = $this->conductorHelper->getAccountId();
    // Check account id if received.
    if (!empty($account_id)) {
      $this->messenger->addMessage(Markup::create('Successfully established connection to Conductor API.'));
    }
    else {
      $this->messenger
        ->addError(Markup::create('Unable to connect to Conductor API. Please check credentials or <a href="https://app.conductor.com/u/password/forgot" target="_blank">reset your password</a>'));
    }

    return parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'conductor.settings',
    ];
  }

}
