<?php

namespace Drupal\conductor\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Theme\ThemeInitializationInterface;
use Drupal\Core\Theme\ThemeManagerInterface;
use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;
use Psr\Log\LoggerInterface;

/**
 * Provides Conductor API helper services.
 *
 * @package Drupal\conductor\Service\ConductorHelper
 */
class ConductorHelper {

  use StringTranslationTrait;

  /**
   * Conductor Helper.
   *
   * @var \Drupal\conductor\Service\ConductorConnector
   */
  protected $conductorConnector;

  /**
   * Entity Type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Module Extension List.
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected $moduleBasePath;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * Logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Theme Initialization.
   *
   * @var \Drupal\Core\Theme\ThemeInitializationInterface
   */
  protected ThemeInitializationInterface $theme;

  /**
   * Theme Manager.
   *
   * @var \Drupal\Core\Theme\ThemeManagerInterface
   */
  protected ThemeManagerInterface $themeManager;

  /**
   * Constructs class variables.
   *
   * @param \Drupal\conductor\Service\ConductorConnector $conductor_connector
   *   Conductor Connector service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Extension\ModuleExtensionList $module_extension
   *   Module Extension List.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_service
   *   The configuration service.
   * @param \Psr\Log\LoggerInterface $logger
   *   Drupal Logger Factory.
   * @param \Drupal\Core\Theme\ThemeInitializationInterface $theme
   *   Theme initialization service.
   * @param \Drupal\Core\Theme\ThemeManagerInterface $theme_manager
   *   Theme manager service.
   */
  public function __construct(
    ConductorConnector $conductor_connector,
    EntityTypeManagerInterface $entity_type_manager,
    ModuleExtensionList $module_extension,
    RendererInterface $renderer,
    ConfigFactoryInterface $config_service,
    LoggerInterface $logger,
    ThemeInitializationInterface $theme,
    ThemeManagerInterface $theme_manager,
  ) {
    $this->conductorConnector = $conductor_connector;
    $this->entityTypeManager = $entity_type_manager;
    $this->moduleBasePath = $module_extension->getPath('conductor');
    $this->renderer = $renderer;
    $this->config = $config_service;
    $this->logger = $logger;
    $this->theme = $theme;
    $this->themeManager = $theme_manager;
  }

  /**
   * Get Conductor Account ID.
   */
  public function getAccountId() {
    $account = $this->conductorConnector->request('GET', '/accounts');
    $account_id = '';
    if (is_array($account)) {
      $account_id = $account[0]['accountId'] ?? '';
    }

    return $account_id;
  }

  /**
   * Get Conductor Web Property ID.
   *
   * @param int $account_id
   *   Conductor Account ID.
   */
  public function getWebPropertyId($account_id) {
    $web_property = $this->conductorConnector->request('GET', '/accounts/' . $account_id . '/web-properties');
    $web_property_id = '';
    if (!empty($web_property)) {
      $web_property_id = $web_property[0]['webPropertyId'] ?? '';
    }

    return $web_property_id;
  }

  /**
   * Generate Recommendations.
   *
   * @param int $account_id
   *   Conductor Account ID.
   * @param string $request_id
   *   Content Guidance Request ID.
   * @param string $override_api_key
   *   Override for config API Key.
   * @param string $override_shared_secret
   *   Override for config Shared Secret.
   */
  public function fetchRecommendations($account_id, $request_id, $override_api_key = '', $override_shared_secret = '') {
    $data = [];
    $url = '/' . $account_id . '/content-guidance?requestUuid=' . $request_id;
    if (!empty($request_id)) {
      // Get Recommendations.
      $data = $this->conductorConnector->request('GET', $url, [], $override_api_key, $override_shared_secret);
      // Log request.
      if ($this->config->get('conductor.settings')->get('conductor.logging')) {
        $response = [
          'account_id' => $account_id,
          'request_id' => $request_id,
          'request URL' => $url,
          'override_api_key' => $override_api_key,
          'override_shared_secret' => $override_shared_secret,
          'recommendations' => $data,
        ];
        if (isset($data['progress']['error']) && $data['progress']['error'] > 0) {
          $this->logger->error('<pre>Fetch Recommendations API: ' . print_r($response, TRUE) . '</pre>');
        }
        else {
          $this->logger->debug('<pre>Fetch Recommendations API: ' . print_r($response, TRUE) . '</pre>');
        }
      }
    }
    else {
      $this->logger->error('Fetch Recommendations API: Request ID is empty, fetch failed.');
    }

    return $data;
  }

  /**
   * Fetch Request ID.
   *
   * @param int $account_id
   *   Conductor Account ID.
   * @param int $web_property_id
   *   Conductor Web Property ID.
   * @param string $topic
   *   Topic of request.
   * @param int $rank_source_id
   *   Conductor Rank Source ID.
   * @param string $url
   *   Page URL.
   * @param string $body
   *   Body copy of page.
   * @param string $override_api_key
   *   Override for config API Key.
   * @param string $override_shared_secret
   *   Override for config Shared Secret.
   */
  public function fetchRequestId($account_id, $web_property_id, $topic, $rank_source_id = 1, $url = '', $body = '', $override_api_key = '', $override_shared_secret = '') {
    $page = [];
    // Page param check/construction.
    if (!empty($url)) {
      $page['url'] = $url;
    }
    if (!empty($body)) {
      $page['bodyCopy'] = $body;
    }
    // Construct params.
    $params = [
      'body' => json_encode([
        'input' => [
          'page' => $page,
          'topic' => [
            'phrase' => $topic,
            'rankSourceId' => $rank_source_id,
            'webPropertyId' => $web_property_id,
            'webPropertyName' => '',
          ],
        ],
        'request' => [
          'insights' => [
            "CONTENT_SCORE_SNIPPET",
            "PRIMARY_KEYWORDS_INSIGHT",
            "SECONDARY_KEYWORDS_INSIGHT",
            "HTML_TITLE_COMMON",
            "HTML_META_DESCRIPTION_COMMON",
            "HTML_HEADING1_COMMON",
            "HTML_HEADING2_COMMON",
            "HTML_HEADING3_COMMON",
            "CONTENT_ANALYSIS_INSIGHT",
            "CONTENT_LENGTH_INSIGHT",
            "READABILITY_INSIGHT",
            "QUESTIONS_INSIGHT_V2",
            "CANNIBALIZATION_INSIGHT",
            "HTML_IMAGE_ALT_COMMON",
            "SCHEMA_MARKUP_INSIGHT",
          ],
        ],
      ]),
    ];

    $request_id = $this->conductorConnector->request('POST', '/' . $account_id . '/content-guidance', $params, $override_api_key, $override_shared_secret);
    $request_id = !empty($request_id['request_id']) ? $request_id['request_id'] : '';

    // Log request.
    if ($this->config->get('conductor.settings')->get('conductor.logging')) {
      $request = [
        'account_id' => $account_id,
        'request_id' => $request_id,
        'request_parameters' => $params,
      ];
      $this->logger->debug('<pre>Fetch RequestID API: ' . print_r($request, TRUE) . '</pre>');
    }

    return $request_id;
  }

  /**
   * Check content type validation.
   *
   * @param object $form_state
   *   The current state of the form.
   * @param string $form_id
   *   A string that is the unique ID of the form.
   */
  public function checkContentTypeValidation($form_state, $form_id) {
    if (substr($form_id, 0, 5) !== 'node_' || substr($form_id, -5) !== '_form') {
      // Not a node form `node_..._form`.
      return FALSE;
    }
    /** @var \Drupal\node\NodeInterface $node */
    $node = $form_state->getFormObject()->getEntity();
    if (!$node instanceof NodeInterface) {
      // Ensure we have the Node entity to proceed.
      return FALSE;
    }
    $bundle = $node->bundle();
    $config = $this->config->get('conductor.settings');
    $content_types = !empty($config->get('conductor.content_types')) ? $config->get('conductor.content_types') : [];
    if (!in_array($bundle, $content_types)) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Get list of content types.
   */
  public function getContentTypes() {
    $types = [];
    $content_types = $this->entityTypeManager
      ->getStorage('node_type')
      ->loadMultiple();

    foreach ($content_types as $name => $type) {
      $types[$name] = !empty($type->get('name')) ? $type->get('name') : '';
    }

    return $types;
  }

  /**
   * Get list of Rank Sources.
   */
  public function getRankSources() {
    $rank_sources = [];
    $account_id = $this->getAccountId() ?? '';
    if (empty($account_id)) {
      return $rank_sources;
    }

    $rank_source_data = $this->conductorConnector->request('GET', '/' . $account_id . '/rank-sources/content-guidance') ?? [];
    foreach ($rank_source_data as $source) {
      if (empty($source)) {
        continue;
      }
      $rank_sources[$source['id']] = $source['description'];
    }
    asort($rank_sources);

    return $rank_sources;
  }

  /**
   * Format number from 1000 to 1k.
   *
   * @param int $number
   *   Number to convert.
   */
  public function convertNumberToK($number) {
    if ($number >= 1000000) {
      return $number / 1000000 . " M";
    }
    elseif ($number >= 1000) {
      return $number / 1000 . " K";
    }
    else {
      return $number;
    }
  }

  /**
   * Convert decimal to percent.
   *
   * @param int $number
   *   Number to convert.
   */
  public function convertToPercent($number) {
    return floor((float) $number * 100) . '%';
  }

  /**
   * Construct guidance status class.
   *
   * @param array $data
   *   Endpoints data array.
   */
  public function constructGuidanceStatusClass($data) {
    $status_mapping = [
      $data['status'] ?? '',
      $data['insightsStatus'] ?? '',
      $data['snippetsStatus'] ?? '',
    ];
    $guidance_status = '';
    if (in_array('PROCESSING', $status_mapping)) {
      $guidance_status = 'processing';
    }
    elseif (in_array('COMPLETED_WITH_ERROR', $status_mapping) || in_array('FAILED', $status_mapping)) {
      $guidance_status = 'failed';
    }
    elseif (in_array('COMPLETED', $status_mapping)) {
      $guidance_status = 'completed';
    }

    return $guidance_status;
  }

  /**
   * Construct keyword insights table row.
   *
   * @param string $type
   *   Result type.
   * @param string $summary
   *   Result summary.
   * @param array $keywords
   *   Keyword array.
   */
  public function constructKeywordsInsight($type, $summary, $keywords) {
    $label = '';
    $build = [
      '#theme' => 'table',
      '#header' => [
        [
          'data' => $this->t('Keyword'),
          'scope' => 'col',
        ],
        [
          'data' => $this->t('Volume'),
          'scope' => 'col',
        ],
        [
          'data' => $this->t('Your draft'),
          'scope' => 'col',
        ],
      ],
    ];
    $error_message = $this->errorMessage();
    if ($type == 'PRIMARY_KEYWORDS_INSIGHT') {
      $label = $this->t('Primary keywords to target');
    }
    elseif ($type == 'SECONDARY_KEYWORDS_INSIGHT') {
      $label = $this->t('Secondary keywords to target');
    }
    // Construct Keywords.
    $keywords = $keywords['details']['keywords']['items'] ?? [];
    $i = 0;
    foreach ($keywords as $keyword) {
      $draft = !empty($keyword['ownRank']) ? $this->t('Yes') : $this->t('No');
      $volume = !empty($keyword['msv']) ? $this->convertNumberToK($keyword['msv']) : '';
      $class = ($i > 4) ? 'hidden-row hidden' : '';
      $build['#rows'][] = [
        'data' => [
          [
            'data' => $keyword['keyword'],
          ],
          [
            'data' => $volume,
          ],
          [
            'data' => $draft,
          ],
        ],
        'class' => $class,
      ];
      $i++;
    }
    // Append see more/less if more than 5.
    if ($i > 5) {
      $build['#rows'][] = $this->seeMore();
      $build['#rows'][] = $this->seeLess();
    }

    // Error message.
    if ($i == 0) {
      $build['#rows'][] = [
        'data' => [
          [
            'data' => ['#markup' => $error_message],
            'colspan' => 3,
            'class' => 'conductor__error',
          ],
        ],
      ];
      $build['#header'] = [];
    }
    $build = $this->renderer->render($build);

    // Construct HTML.
    $html = [
      '#markup' => '
      <div class="conductor__accordion-content__container">
        <h4 class="conductor__accordion-content__label">' . $label . '</h4>
        <p class="conductor__accordion-content__summary">' . $summary . '</p>
        <div class="conductor__accordion-content__table">
          ' . $build . '
        </div>
      </div>
    ',
    ];

    return $this->renderer->render($html);
  }

  /**
   * Construct pattern insights table row.
   *
   * @param string $type
   *   Result type.
   * @param string $summary
   *   Result summary.
   * @param array $keywords
   *   Keyword array.
   */
  public function constructPatternsInsight($type, $summary, $keywords) {
    $error_message = $this->errorMessage();
    $header_mapping = [
      'HTML_HEADING1_COMMON' => [
        'heading' => 'H1',
        'label' => $this->t('H1 Patterns'),
      ],
      'HTML_HEADING2_COMMON' => [
        'heading' => 'H2',
        'label' => $this->t('H2 Patterns'),
      ],
      'HTML_HEADING3_COMMON' => [
        'heading' => 'H3',
        'label' => $this->t('H3 Patterns'),
      ],
      'HTML_TITLE_COMMON' => [
        'heading' => 'title',
        'label' => $this->t('Title tag patterns'),
      ],
      'HTML_META_DESCRIPTION_COMMON' => [
        'heading' => 'description',
        'label' => $this->t('Meta description patterns'),
      ],
    ];
    $heading = $this->t('Your') . ' ' . $header_mapping[$type]['heading'];
    $build = [
      '#theme' => 'table',
      '#header' => [
        [
          'data' => $this->t('Keyword'),
          'scope' => 'col',
        ],
        [
          'data' => $this->t('Ranking Pages'),
          'scope' => 'col',
        ],
        [
          'data' => $heading,
          'scope' => 'col',
        ],
      ],
    ];
    // Construct Keywords.
    $total = $keywords['details']['keywords']['numOfSerps'] ?? '';
    $keywords = $keywords['details']['keywords']['items'] ?? [];
    $i = 0;
    foreach ($keywords as $keyword) {
      $your_pattern = $keyword['ownMatched'] ? $this->t('Yes') : $this->t('No');
      $class = ($i > 4) ? 'hidden-row hidden' : '';
      $build['#rows'][] = [
        'data' => [
          [
            'data' => $keyword['keyword'],
          ],
          [
            'data' => $keyword['numOfOccurrence'] . '/' . $total,
          ],
          [
            'data' => $your_pattern,
          ],
        ],
        'class' => $class,
      ];
      $i++;
    }
    // Append see more/less if more than 5.
    if ($i > 4) {
      $build['#rows'][] = $this->seeMore();
      $build['#rows'][] = $this->seeLess();
    }
    // Error message.
    if ($i == 0) {
      $build['#rows'][] = [
        'data' => [
          [
            'data' => ['#markup' => $error_message],
            'colspan' => 3,
            'class' => 'conductor__error',
          ],
        ],
      ];
      $build['#header'] = [];
    }
    $build = $this->renderer->render($build);

    // Construct HTML.
    $html = [
      '#markup' => '
      <div class="conductor__accordion-content__container">
        <h4 class="conductor__accordion-content__label">' . $header_mapping[$type]['label'] . '</h4>
        <p class="conductor__accordion-content__summary">' . str_replace(':.', ':', $summary) . '</p>
        <div class="conductor__accordion-content__table">
          ' . $build . '
        </div>
      </div>
    ',
    ];

    return $this->renderer->render($html);
  }

  /**
   * Construct Content Optimization Body Copy table row.
   *
   * @param array $keywords
   *   Keywords array.
   */
  public function constructContentOptimizationBodyCopy($keywords) {
    $presets = [
      'IMPORTANT',
      'SUPPORTING',
      'ADDITIONAL',
    ];
    $error_message = $this->errorMessage();
    // Construct Keywords.
    $total = $keywords['details']['keywords']['maxOccurrence'] ?? '';
    $factors = $keywords['details']['predictiveFactors'] ?? [];
    $keywords = $keywords['details']['keywords']['items'] ?? [];
    $tables = [];
    // Construct Summary.
    $summary = [];
    foreach ($factors as $factor) {
      $summary[] = $factor['message'];
    }
    $summary = implode(' ', $summary);
    // Preset grouping.
    $body_copy_array = [];
    foreach ($presets as $preset) {
      $body_copy_array[$preset] = array_filter($keywords, function ($keyword) use ($preset) {
        return array_values($keyword['presets'])[0] === $preset;
      });
    }
    // Keyword rows.
    foreach ($body_copy_array as $preset_key => $keywords_data) {
      $build = [
        '#theme' => 'table',
        '#attributes' => ['class' => 'conductor__body-copy'],
      ];
      // Help text message.
      switch ($preset_key) {
        case 'IMPORTANT':
          $label = $this->t('Important Keywords');
          $text = $this->t('Most of the top 15 ranking pages for your target topic use these semantically relevant terms.');
          break;

        case 'SUPPORTING':
          $label = $this->t('Supporting Keywords');
          $text = $this->t('Many (but not all) of your SEO competitors use these terms in their pages ranking for your target topic. They are semantically relevant, but not as relevant as the Important keywords.');
          break;

        case 'ADDITIONAL':
          $label = $this->t('Additional Keywords');
          $text = $this->t('Some of your SEO competitors use these terms in their pages ranking for your target topic. They are semantically associated with your target topic, but they may not be directly relevant.');
          break;

        default:
          $label = '';
          $text = '';
      }
      $help_text = '<p><b>' . $label . ':</b></p><p>' . $text . '</p>';
      // Preset header mapping.
      $header_html = '
        <h5>' . ucfirst(strtolower($preset_key)) . '</h5>
        <img src="/' . strval($this->moduleBasePath) . '/images/help-circle.svg" />
        <div class="conductor__help-text__popup">
          <div class="conductor__help-text__popup__text">' . $help_text . '</div>
        </div>
      ';
      $build['#rows'][] = [
        'data' => [
          [
            'data' => ['#markup' => $header_html],
            'colspan' => 3,
            'class' => 'conductor__help-text__container',
          ],
        ],
      ];
      $build['#rows'][] = [
        'data' => [
          [
            'data' => $this->t('Term'),
          ],
          [
            'data' => $this->t('Ranking pages'),
          ],
          [
            'data' => $this->t('Your draft'),
          ],
        ],
        'class' => 'conductor__body-copy__header',
      ];
      // Inner table rows.
      $i = 0;
      foreach ($keywords_data as $keyword) {
        $draft = !empty($keyword['ownFrequency']) ? $this->t('Yes') : $this->t('No');
        $class = ($i > 4) ? 'hidden-row hidden' : '';
        $build['#rows'][] = [
          'data' => [
            [
              'data' => $keyword['keyword'],
            ],
            [
              'data' => $keyword['numOfOccurrence'] . '/' . $total,
            ],
            [
              'data' => $draft,
            ],
          ],
          'class' => $class,
        ];
        $i++;
      }
      // Append see more/less if more than 5.
      if ($i > 4) {
        $build['#rows'][] = $this->seeMore();
        $build['#rows'][] = $this->seeLess();
      }
      // Error message.
      if ($i == 0) {
        $build['#rows'][] = [
          'data' => [
            [
              'data' => ['#markup' => $error_message],
              'colspan' => 3,
              'class' => 'conductor__error',
            ],
          ],
        ];
      }
      $tables[] = $this->renderer->render($build);
    }

    // Construct HTML.
    $html = [
      '#markup' => '
      <div class="conductor__accordion-content__container">
        <h4 class="conductor__accordion-content__label">' . $this->t('Body copy suggestions') . '</h4>
        <p class="conductor__accordion-content__summary">' . $summary . '</p>
        <div class="conductor__accordion-content__table">' . implode('', $tables) . '</div>
      </div>
    ',
    ];

    return $this->renderer->render($html);
  }

  /**
   * Construct Content Score.
   *
   * @param array $details
   *   Snippet details array.
   */
  public function constructContentScore($details) {
    $html = ['#markup' => ''];
    if (!empty($details['content_score'])) {
      $title = $this->t('Content Score');
      $help_text_description = $this->t('Reflects how holistically and comprehensively your content addresses the given topic. Higher scores reflect higher potential for your content to perform successfully in search results.');
      $help_text_list = $this->t('It is based on three components');
      $keyword = $this->t('Keyword coverage');
      $sentence = $this->t('Sentence structure');
      $repetition = $this->t('Repetitions');
      $html = [
        '#markup' => '
          <div class="conductor__content-score">
            <div class="conductor__help-text__container">
              <p>' . $title . ': <b>' . $this->convertToPercent($details['content_score']) . '</b></p>
              <img src="/' . strval($this->moduleBasePath) . '/images/help-circle.svg" />
              <div class="conductor__help-text__popup">
                <div class="conductor__help-text__popup__text">
                  <p>' . $help_text_description . '</p>
                  <p>' . $help_text_list . ':</p>
                  <ul>
                    <li>' . $keyword . '</li>
                    <li>' . $sentence . '</li>
                    <li>' . $repetition . '</li>
                  </ul>
                </div>
              </div>
            </div>
          </div>
      ',
      ];
    }

    return $this->renderer->render($html);
  }

  /**
   * Construct Score Factors.
   *
   * @param array $keywords
   *   Keywords array.
   */
  public function constructScoreFactors($keywords) {
    $keyword_rows = '';
    $error_message = '<div class="conductor__error"><p><b>' . $this->t('Could not generate guidance at this time.') . '</b> ' . $this->t('Please try again later.') . '</p></div>';
    // Detail Mapping.
    $keyword = $this->t('Keyword coverage');
    $sentence = $this->t('Sentence structure');
    $repetition = $this->t('Repetitions');
    $detail_mapping = [
      'coverage_score' => [
        'label' => '<p>' . $keyword . ': <b>' . $this->convertToPercent($keywords['coverage_score']) . '</b></p>',
        'help_text' => '<p>' . $this->t('Calculates your keyword coverage, based on our Body Copy Suggestions. The more recommended keywords you add in your text, the higher the score.') . '</p>',
      ],
      'natural_language_score' => [
        'label' => '<p>' . $sentence . ': <b>' . $this->convertToPercent($keywords['natural_language_score']) . '</b></p>',
        'help_text' => '<p>' . $this->t('Assesses how well your text is structured. Our deep learning algorithm detects when your content does not use natural language, and calculates the score accordingly. For example, if you write a phrase like "I eats fish", the algorithm will detect the incorrect usage and lower your score.') . '</p>',
      ],
      'repetition_score' => [
        'label' => '<p>' . $repetition . ': <b>' . $this->convertToPercent($keywords['repetition_score']) . '</b></p>',
        'help_text' => '<p>' . $this->t('Assesses how often you repeat text in your content. If you are heavily repeating sentences and phrases, your score will be negatively impacted. This does not apply to individual words—only phrases and sentences.') . '</p>',
      ],
    ];

    foreach ($detail_mapping as $detail) {
      $keyword_rows .= '
        <div class="conductor__help-text__container conductor__score-factors">
          ' . $detail['label'] . '
          <img src="/' . strval($this->moduleBasePath) . '/images/help-circle.svg" />
          <div class="conductor__help-text__popup">
            <div class="conductor__help-text__popup__text">
              ' . $detail['help_text'] . '
            </div>
          </div>
        </div>
      ';
    }

    if (empty($keywords['repetition_score']) && empty($keywords['natural_language_score']) && empty($keywords['coverage_score'])) {
      $keyword_rows .= $error_message;
    }

    $html = [
      '#markup' => '
      <div class="conductor__accordion-content__container">
        <h4 class="conductor__accordion-content__label">' . $this->t('Score factors') . '</h4>
        <p class="conductor__accordion-content__summary">' . $this->t('Make sure you are covering the most important topics from the Body Copy suggestions in this draft.') . '</p>
        <div class="conductor__accordion-content__table">' . $keyword_rows . '</div>
      </div>
    ',
    ];

    return $this->renderer->render($html);
  }

  /**
   * Construct Questions to answer.
   *
   * @param array $keywords
   *   Keywords array.
   */
  public function constructQuestions($keywords) {
    $build = [
      '#theme' => 'table',
    ];
    $error_message = '<div class="conductor__accordion-content__summary conductor__error"><p>' . $this->t('Could not generate AI advice for the following questions. Error connecting to OpenAI. Please hit the “Refresh Guidance” button to try again.') . '</p></div>';
    $summary = '<p class="conductor__accordion-content__summary">' . $this->t('Discover how your draft can better answer the questions your audience is asking about this topic. Helpful content is more valuable to the customer and often ranks higher.') . '</p>';
    $items = $keywords['details']['paaQuestions']['items'] ?? [];
    if (!empty($items)) {
      $i = 0;
      foreach ($items as $item) {
        $question = $item['text'] ?? '';
        $answer = $item['answer'] ?? '';
        $class = ($i > 1) ? 'hidden-row hidden' : '';
        $answer = !empty($answer) ? '<p class="conductor__accordion-content__summary"><i>' . $answer . '</i></p>' : '';
        $build['#rows'][] = [
          'data' => [
            [
              'data' => [
                '#markup' => '<h4 class="conductor__accordion-content__label">' . $question . '</h4>' . $answer,
              ],
              'colspan' => 3,
            ],
          ],
          'class' => $class,
        ];
        $i++;
      }
      // Append see more/less if more than 2.
      if ($i > 1) {
        $build['#rows'][] = $this->seeMore();
        $build['#rows'][] = $this->seeLesS();
      }
    }

    // Set error message.
    if (empty($build['#rows'])) {
      $build['#rows'][] = [
        'data' => [
          [
            'data' => ['#markup' => $error_message],
          ],
        ],
      ];
    }
    $build = $this->renderer->render($build);

    $html = '
      <div class="conductor__accordion-content__container conductor__questions-to-answer">
        <h4 class="conductor__accordion-content__label">Questions to answer</h4>
        ' . $summary . '
        ' . $build . '
      </div>
    ';

    return $html;
  }

  /**
   * Construct Content Length.
   *
   * @param array $keywords
   *   Keywords array.
   */
  public function constructContentLength($keywords) {
    $keyword_rows = '';
    $details = $keywords['details'] ?? [];
    $factors = $details['predictiveFactors'] ?? [];
    $error_message = '<div class="conductor__error"><p><b>' . $this->t('Could not generate guidance at this time.') . '</b> ' . $this->t('Please try again later.') . '</p></div>';
    $metric_value = !empty($details['own']['metricValue']) ? number_format($details['own']['metricValue']) : 0;
    $average_metric_value = !empty($details['serp']['averageMetricValue']) ? number_format($details['serp']['averageMetricValue']) : 0;
    // Construct Summary.
    $summary = [];
    foreach ($factors as $factor) {
      $summary[] = $factor['message'];
    }
    $summary = implode(' ', $summary);

    if (!empty($average_metric_value)) {
      $draft = $this->t('Your draft');
      $average = $this->t('Top pages average');
      $keyword_rows .= '
        <div class="conductor__content-length">
          <p>' . $draft . ':</p>
          <p>' . $metric_value . ' words</p>
        </div>
        <div class="conductor__content-length">
          <p>' . $average . ':</p>
          <p>' . $average_metric_value . ' words</p>
        </div>
      ';
    }
    else {
      $keyword_rows .= $error_message;
    }

    $html = [
      '#markup' => '
      <div class="conductor__accordion-content__container conductor__questions-to-answer">
        <h4 class="conductor__accordion-content__label">' . $this->t('Length') . '</h4>
        <p class="conductor__accordion-content__summary">' . $summary . '</p>
        ' . $keyword_rows . '
      </div>
    ',
    ];

    return $this->renderer->render($html);
  }

  /**
   * Construct Readability Insight.
   *
   * @param string $summary
   *   Result summary.
   * @param array $keywords
   *   Keywords array.
   */
  public function constructReadabilityInsight($summary, $keywords) {
    $keyword_rows = '';
    $details = $keywords['details'] ?? [];
    $error_message = '<div class="conductor__error"><p><b>' . $this->t('Could not generate guidance at this time.') . '</b> ' . $this->t('Please try again later.') . '</p></div>';
    $your_draft = !empty($details['own']['metricValue']['label']) ? $details['own']['metricValue']['label'] : 'N/A';
    $average = $details['serp']['averageMetricValue']['label'] ?? [];

    if (!empty($average)) {
      $keyword_rows .= '
        <div class="conductor__content-readability">
          <p>Your draft:</p>
          <p>' . $your_draft . '</p>
        </div>
        <div class="conductor__content-readability">
          <p>Top pages average:</p>
          <p>' . $average . '</p>
        </div>
      ';
    }
    else {
      $keyword_rows .= $error_message;
    }

    $html = [
      '#markup' => '
      <div class="conductor__accordion-content__container conductor__readability">
        <h4 class="conductor__accordion-content__label">Readability</h4>
        <p class="conductor__accordion-content__summary">' . $summary . '</p>
        ' . $keyword_rows . '
      </div>
    ',
    ];

    return $this->renderer->render($html);
  }

  /**
   * See more Form API.
   */
  private function seeMore() {
    return [
      'data' => [
        [
          'data' => [
            '#markup' => '<a href="#" class="see-more text-center">' . $this->t('See more') . '</a>',
          ],
          'colspan' => 3,
        ],
      ],
      '#attributes' => ['class' => 'see-wrapper'],
    ];
  }

  /**
   * See less Form API.
   */
  private function seeLess() {
    return [
      'data' => [
        [
          'data' => [
            '#markup' => '<a href="#" class="see-less hidden text-center">' . $this->t('See Less') . '</a>',
          ],
          'colspan' => 3,
        ],
      ],
      '#attributes' => ['class' => 'see-wrapper'],
    ];
  }

  /**
   * Error message.
   */
  private function errorMessage() {
    return '<p><b>' . $this->t('Could not generate guidance at this time.') . '</b> ' . $this->t('Please try again later.') . '</p>';
  }

  /**
   * Build body copy.
   *
   * @param \Drupal\node\Entity\Node $node
   *   The Node entity.
   *
   * @throws \Drupal\Core\Theme\MissingThemeDependencyException
   */
  public function buildPreview(Node $node): string {

    // Set the default theme as the current active theme.
    $default_theme = $this->config->get('system.theme')->get('default');
    $default_theme_object = $this->theme->getActiveThemeByName($default_theme);
    $this->themeManager->setActiveTheme($default_theme_object);

    $view_builder = $this->entityTypeManager->getViewBuilder('node');
    $content = $view_builder->view($node, 'full');
    $content['#conductor_uuid'] = $node->uuid();
    $build = [
      '#type' => 'html',
      'page' => [
        '#theme' => 'page',
        'content' => [
          'page_title' => [
            '#type' => 'page_title',
            '#title' => $node->getTitle(),
          ],
          'main_content' => [
            'content' => $content,
          ],
        ],
      ],
    ];

    $value = $this->renderer->renderPlain($build);
    // Reset theme to the admin theme.
    $this->themeManager->resetActiveTheme();

    return $value;
  }

  /**
   * Get guidance generation progress.
   *
   * @param array $data
   *   The data received from the API.
   *
   * @return string
   *   The progress percentage.
   */
  public function getProgress($data): string {
    // Reset.
    if ($data['status'] === 'COMPLETED' || $data['status'] === 'FAILED' || $data['status'] === 'COMPLETED_WITH_ERROR' || empty($data['progress'])) {
      return '0%';
    }

    return round(($data['progress']['completed'] / $data['progress']['total']) * 100) . "%";
  }

}
