<?php

namespace Drupal\Tests\conductor\Kernel;

use Drupal\conductor\Service\ConductorHelper;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\KernelTests\KernelTestBase;
use Drupal\node\Entity\Node;
use Drupal\node\Entity\NodeType;

/**
 * Tests that a node is not saved under certain conditions.
 *
 * @group my_module
 */
class ConductorGenerateRecommendationsTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'user',
    'system',
    'field',
    'text',
    'filter',
    'conductor',
  ];

  /**
   * Sets up the environment for the test.
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installEntitySchema('user');
    $this->installEntitySchema('node');
    $this->installSchema('node', ['node_access']);
    $this->installConfig(['node']);

    // Create a content type.
    NodeType::create(['type' => 'article'])->save();

    // Conductor Helper.
    $conductor_helper = $this->createMock(ConductorHelper::class);

    // Check that the buildPreview method is called.
    $conductor_helper->expects($this->once())
      ->method('buildPreview')
      ->willReturn('New Article');

    // Check that the fetchRequestId method is called.
    $conductor_helper->expects($this->once())
      ->method('fetchRequestId')
      ->withAnyParameters()
      ->willReturn('123');

    // Check that the fetchRecommendations method is called.
    $conductor_helper->expects($this->once())
      ->method('fetchRecommendations')
      ->willReturn([]);

    $this->container->set('conductor.helper', $conductor_helper);
    // Conductor Config.
    $this->container->get('config.factory')->getEditable('conductor.settings')
      ->set('conductor.rank_sources', '')->save();
  }

  /**
   * Tests node is not saved when generating recommendations for new nodes.
   *
   * @throws \Exception
   */
  public function testNodeNotSavedWhenGeneratingRecommendationsForNewNodes(): void {
    // Apply the conditions, here we just prepare a node.
    $node = Node::create([
      'type' => 'article',
      'title' => 'New Article',
    ]);
    $node->setNewRevision();

    $form = [];

    $form_object = $this->createMock(EntityForm::class);
    $form_object->expects($this->once())
      ->method('getEntity')
      ->willReturn($node);

    $form_state = $this->createMock(FormStateInterface::class);
    $form_state->expects($this->once())
      ->method('getFormObject')
      ->willReturn($form_object);

    $form_state->expects($this->atLeastOnce())
      ->method('getValue')
      ->willReturn(NULL, 'Sample topic', [], [], 'Sample topic');

    conductor_generate_recommendations($form, $form_state);

    $node_revisions_count = \Drupal::entityQuery('node')
      ->condition('title', 'New Article')
      ->allRevisions()
      ->count()
      ->accessCheck(TRUE)
      ->execute();
    $this->assertEquals(0, $node_revisions_count, 'No nodes with the title "New Article" were found.');
  }

  /**
   * Tests node is not saved when generating recommendations for existing nodes.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function testNodeNotSavedWhenGeneratingRecommendationsForExistingNodes(): void {
    // Apply the conditions, here we just prepare a node.
    $node = Node::create([
      'type' => 'article',
      'title' => 'New Article',
    ]);

    $node->save();

    // Update the node and run analysis.
    $node->setTitle('New Updated Title');
    $node->setNewRevision();

    $form = [];

    $form_object = $this->createMock(EntityForm::class);
    $form_object->expects($this->once())
      ->method('getEntity')
      ->willReturn($node);

    $form_state = $this->createMock(FormStateInterface::class);
    $form_state->expects($this->once())
      ->method('getFormObject')
      ->willReturn($form_object);

    $form_state->expects($this->atLeastOnce())
      ->method('getValue')
      ->willReturn(1, 'Sample topic', [], [], 'Sample topic');

    conductor_generate_recommendations($form, $form_state);

    $node_revisions_count = \Drupal::entityQuery('node')
      ->condition('uuid', $node->uuid())
      ->allRevisions()
      ->count()
      ->accessCheck(TRUE)
      ->execute();
    $this->assertEquals(1, $node_revisions_count, 'Only 1 node revision were found.');
  }

}
