<?php

namespace Drupal\Tests\conductor\Unit;

use Drupal\conductor\Service\ConductorConnector;
use Drupal\conductor\Service\ConductorHelper;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Theme\ThemeInitializationInterface;
use Drupal\Core\Theme\ThemeManagerInterface;
use Drupal\Tests\UnitTestCase;
use GuzzleHttp\Client;

/**
 * @coversDefaultClass \Drupal\conductor\Service\ConductorHelper
 * @group conductor
 */
class ConductorHelperTest extends UnitTestCase {

  /**
   * Config Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $configSettings;

  /**
   * Guzzle HTTP client.
   *
   * @var \GuzzleHttp\Client|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $httpClient;

  /**
   * Logger.
   *
   * @var \Psr\Log\LoggerInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $logger;

  /**
   * Conductor Connector service.
   *
   * @var \Drupal\conductor\Service\ConductorConnector|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $connectorService;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $entityTypeManager;

  /**
   * The node type storage handler.
   *
   * @var \Drupal\Core\Config\Entity\ConfigEntityStorage|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $nodeTypeStorage;

  /**
   * Entity Storage interface.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $entityStorage;

  /**
   * Module Extension List.
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $moduleExtension;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $renderer;

  /**
   * Theme Initialization Interface.
   *
   * @var \Drupal\Core\Theme\ThemeInitializationInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $theme;

  /**
   * Theme Manager Interface.
   *
   * @var \Drupal\Core\Theme\ThemeManagerInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $themeManager;

  /**
   * Conductor Helper service.
   *
   * @var \Drupal\conductor\Service\ConductorHelper|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $helperService;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->httpClient = $this->createMock(Client::class);
    $this->logger = $this->createMock('\Psr\Log\LoggerInterface');
    $this->configSettings = $this->getConfigFactoryStub([
      'conductor.settings' => [
        'conductor' => [
          'api_key' => '',
          'shared_secret' => '',
          'content_types' => [
            'article',
          ],
        ],
      ],
    ]);
    $this->connectorService = new ConductorConnector($this->configSettings, $this->httpClient, $this->logger);
    $this->entityTypeManager = $this->createMock(EntityTypeManagerInterface::class);
    $this->nodeTypeStorage = $this->createMock('\Drupal\Core\Config\Entity\ConfigEntityStorage');
    $this->entityStorage = $this->getMockBuilder(EntityStorageInterface::class)
      ->disableOriginalConstructor()
      ->getMock();
    $this->entityStorage->expects($this->any())
      ->method('loadMultiple')
      ->willReturn($this->nodeTypeStorage);
    $this->entityTypeManager->expects($this->any())
      ->method('getStorage')
      ->with('node_type')
      ->willReturn($this->entityStorage);
    $this->moduleExtension = $this->createMock(ModuleExtensionList::class);
    $this->renderer = $this->createMock('\Drupal\Core\Render\RendererInterface');
    $this->theme = $this->createMock(ThemeInitializationInterface::class);
    $this->themeManager = $this->createMock(ThemeManagerInterface::class);
    $this->helperService = new ConductorHelper($this->connectorService, $this->entityTypeManager, $this->moduleExtension, $this->renderer, $this->configSettings, $this->logger, $this->theme, $this->themeManager);
  }

  /**
   * Tests getting account ID.
   */
  public function testGetAccountId() {
    // Get account ID.
    $account_id = $this->helperService->getAccountId();
    // Request will return an empty array unless provided a valid api key and
    // shared secret.
    $this->assertEmpty($account_id);
  }

  /**
   * Tests getting web property ID.
   */
  public function testGetWebPropertyId() {
    // Get Web Property ID.
    $web_property_id = $this->helperService->getWebPropertyId('account_id');
    // Request will return an empty array unless provided a valid api key and
    // shared secret.
    $this->assertEmpty($web_property_id);
  }

  /**
   * Tests fetching recommendations.
   */
  public function testFetchRecommendations() {
    // Fetch recommendations.
    $recommendations = $this->helperService->fetchRecommendations('account_id', 'request_id', $override_api_key = '', $override_shared_secret = '');
    // Request will return an empty array unless provided a valid api key and
    // shared secret.
    $this->assertEmpty($recommendations);
  }

  /**
   * Tests fetching the request ID.
   */
  public function testFetchRequestId() {
    // Fetch request ID.
    $request_id = $this->helperService->fetchRequestId('account_id', 'request_id', 'topic', 1, $url = '', $body = '', $override_api_key = '', $override_shared_secret = '');
    // Request will return an empty array unless provided a valid api key and
    // shared secret.
    $this->assertEmpty($request_id);
  }

  /**
   * TEsts check content type validation.
   */
  public function testCheckContentTypeValidation() {
    $checker = $this->helperService->checkContentTypeValidation((object) [], '_node_edit_form');
    $this->assertFalse($checker);
  }

  /**
   * Tests getting content types.
   */
  public function testGetContentTypes() {
    $content_types = $this->helperService->getContentTypes();
    $this->assertEmpty($content_types);
  }

  /**
   * Tests get rank sources.
   */
  public function testGetRankSources() {
    // Get rank resources.
    $rank_resources = $this->helperService->getRankSources();
    // Request will return an empty array unless provided a valid api key and
    // shared secret.
    $this->assertEmpty($rank_resources);
  }

  /**
   * Tests convert number to thousands.
   */
  public function testConvertNumberToK() {
    // Convert Number.
    $converted_number = $this->helperService->convertNumberToK(1000);
    $this->assertEquals('1 K', $converted_number);
  }

  /**
   * Test convert to percent.
   */
  public function testConvertToPercent() {
    // Convert Number.
    $converted_number = $this->helperService->convertToPercent(0.10);
    $this->assertEquals('10%', $converted_number);
  }

  /**
   * Test instantiation of GuidanceStatus class.
   */
  public function testConstructGuidanceStatusClass() {
    $guidanceStatus = $this->helperService->constructGuidanceStatusClass(['status' => 'COMPLETED']);
    $this->assertEquals('completed', $guidanceStatus);
  }

}
